Package: cpvSNP
Type: Package
Title: Gene set analysis methods for SNP association p-values that lie
        in genes in given gene sets
Version: 1.44.0
Date: 2021-11-20
Author: Caitlin McHugh, Jessica Larson, and Jason Hackney
Maintainer: Caitlin McHugh <mchughc@uw.edu>
Imports: methods, corpcor, BiocParallel, ggplot2, plyr
Depends: R (>= 3.5.0), GenomicFeatures, GSEABase (>= 1.24.0)
Suggests: TxDb.Hsapiens.UCSC.hg19.knownGene, RUnit, BiocGenerics,
        ReportingTools, BiocStyle
Description: Gene set analysis methods exist to combine SNP-level
        association p-values into gene sets, calculating a single
        association p-value for each gene set. This package implements
        two such methods that require only the calculated SNP p-values,
        the gene set(s) of interest, and a correlation matrix (if
        desired). One method (GLOSSI) requires independent SNPs and the
        other (VEGAS) can take into account correlation (LD) among the
        SNPs. Built-in plotting functions are available to help users
        visualize results.
License: Artistic-2.0
biocViews: Genetics, StatisticalMethod, Pathways, GeneSetEnrichment,
        GenomicVariation
git_url: https://git.bioconductor.org/packages/cpvSNP
git_branch: RELEASE_3_23
git_last_commit: 78868a5
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 21:26:26 UTC; biocbuild

import(GSEABase)
import(GenomicFeatures)
import(corpcor)
import(plyr)
import(BiocParallel)
import(ggplot2)
import(methods)

export(assocPvalBySetPlot)
export(createArrayData)
export(geneToSNPList)
export(glossi)
export(glossiMarginal)
export(plotPvals)
export(simulate_chisq)
export(vegas)
export(vegasMarginal)

exportClasses(GeneSetResult, GeneSetResultCollection, VEGASResult, VEGASResultCollection, GLOSSIResult, GLOSSIResultCollection)
exportMethods(geneSetName, pValue, simulatedStats, degreesOfFreedom, statistic)

##' Translates a set of gene ids to their corresponding SNPs.
##'
##' Translates a GeneSet or GeneSetCollection object of gene 
#' Entrez ids to the corresponding SNPs that lie within a 
#' prespecified region of the gene.
##' @title Translate a List of Gene Ids to Their Corresponding 
#' SNP Ids
##' @param geneList An object of class \code{GeneSetCollection}
#'  or \code{GeneSet} where geneIds holds the gene Entrez ids.
##' @param arrayData A \code{GRanges} object that corresponds to 
#' the probes for which data are present. 
##' @param genes A \code{GRanges} object holding the gene transcripts
#' fow which the mapping will occur. The ids of the genes should match
#' the ids that are listed in the \code{geneList} object.
##' @param maxgap An integer value indicating the flanking region 
#' around the gene for which SNPs are considered part of the gene. 
#' Default is 20000 (20kb). 
##' @param quiet A logical variable indicating whether warnings should
#' be output from the SNP to gene mapping step.
##' @return A \code{GeneSetCollection} object containing all SNP ids that 
#' lie within the genes listed in geneList.
##' @author Jason Hackney, Jessica Larson, 
#' Caitlin McHugh \email{mchughc@@uw.edu}
##' @export
geneToSNPList <- function(geneList, arrayData, genes,
    maxgap=20000, quiet=TRUE)
{
    if(!is(genes, "GRanges")){stop("genes must be a GRanges object.")}
	
	if(!is.logical(quiet)){stop("quiet must be a logical argument.")}
	
    if(quiet){
        hitsRes <- suppressWarnings(findOverlaps(genes, arrayData, 
        maxgap = maxgap))
    }else{hitsRes <- findOverlaps(genes, arrayData, maxgap = maxgap)}
    
    hitsMat <- as.data.frame(hitsRes)
    spl <- split(hitsMat$subjectHits,names(genes)[hitsMat$queryHits])

    # this transforms the elements of the split from indices to the snp at 
        # that index
    spl2 <- bplapply(spl, 
                     function(x, arrayData) { arrayData$SNP[x] }, 
                     arrayData=arrayData
            )

    # this creates a gene set collection object that maps identifiers from 
    # the geneList to the AnnotationIdentifier() from the spl2 list 
    snp.gsc <- 
        GeneSetCollection(bplapply(geneList, 
            function(what, to, from, ...) {
                library(GSEABase)
                mapIdentifiers(what, to, from, ...)
            }, to=AnnotationIdentifier(), from=as.environment(spl2)))
    
    return(snp.gsc)
}

# unit tests for the cpvSNP package

test_createArrayData <- function(){
	data(geneSetAnalysis)
	arrayData <- geneSetAnalysis[["arrayData"]]
	checkTrue(class(createArrayData(arrayData,positionName="Position"))=="GRanges")
}

test_geneToSNPList <- function(){
	data(geneSetAnalysis)
	data(test_genesHg19)
	
	geneList <- geneSetAnalysis[["geneSets"]]
	arrayData <- geneSetAnalysis[["arrayData"]]
	arrayDataGR <- createArrayData(geneSetAnalysis[["arrayData"]],
	    position="Position", verbose=FALSE)

	checkException(geneToSNPList(geneList,arrayDataGR,test_genesHg19,maxgap=NA))
	checkException(geneToSNPList(geneList,arrayDataGR,test_genesHg19,maxgap=TRUE))
	checkException(geneToSNPList(geneList,arrayDataGR,test_genesHg19,maxgap="yes"))	
	
	checkException(geneToSNPList(geneList,arrayDataGR,test_genesHg19,quiet=NA))
	checkException(geneToSNPList(geneList,arrayDataGR,test_genesHg19,quiet=12))
	checkException(geneToSNPList(geneList,arrayDataGR,test_genesHg19,quiet="yes"))	
		
	checkTrue(class(geneToSNPList(geneList,arrayDataGR,test_genesHg19))=="GeneSetCollection")
}

test_glossi <- function(){
	data(geneSetAnalysis)
    data(test_snpsGSC)

	pvals <- geneSetAnalysis[["arrayData"]]$P
	names(pvals) <- geneSetAnalysis[["arrayData"]]$SNP
	
	gRes <- glossi(pvals, snpsGSC[[1]])
	checkTrue(class(pValue(gRes))=="numeric")
	checkTrue(class(degreesOfFreedom(gRes))=="integer")
	checkTrue(class(statistic(gRes))=="numeric")
	
	checkEqualsNumeric(degreesOfFreedom(gRes),607)
	
	gRes <- glossi(pvals, snpsGSC)
	checkTrue(length(pValue(gRes))==2)
	checkTrue(length(degreesOfFreedom(gRes))==2)
	checkTrue(length(statistic(gRes))==2)
}

test_vegas <- function(){
	data(geneSetAnalysis)
    data(test_snpsGSC)
	
	arrayDataGR <- createArrayData(geneSetAnalysis[["arrayData"]],
	    position="Position", verbose=FALSE)
	vRes <- vegas(snpsGSC[[1]], arrayDataGR, geneSetAnalysis[["ldMat"]])
	
	# check output
	checkTrue(class(vRes)=="VEGASResult")
	checkTrue(class(pValue(vRes))=="numeric")
	checkTrue(class(statistic(vRes))=="numeric")
	checkTrue(class(degreesOfFreedom(vRes))=="integer")
	checkTrue(length(simulatedStats(vRes))==1000)	
	
	# checking input errors/exceptions
	
	checkException(vegas(snpsGSC[[1]], arrayDataGR, geneSetAnalysis[["ldMat"]], num_sims=0))
	checkException(vegas(snpsGSC[[1]], arrayDataGR, geneSetAnalysis[["ldMat"]], num_sims=NA))
	checkException(vegas(snpsGSC[[1]], arrayDataGR, geneSetAnalysis[["ldMat"]],
	    num_sims=TRUE))
	checkException(vegas(snpsGSC[[1]], arrayDataGR, geneSetAnalysis[["ldMat"]],
	    num_sims="yes"))

	checkException(vegas(NA, arrayDataGR, geneSetAnalysis[["ldMat"]]))
	checkException(vegas("yes", arrayDataGR, geneSetAnalysis[["ldMat"]]))
	checkException(vegas(12, arrayDataGR, geneSetAnalysis[["ldMat"]]))
	checkException(vegas(TRUE, arrayDataGR, geneSetAnalysis[["ldMat"]]))
	checkException(vegas(geneSet(c(0,0)), arrayDataGR, geneSetAnalysis[["ldMat"]]))
 
	checkException(vegas(snpsGSC[[1]], "yes", geneSetAnalysis[["ldMat"]]))
	checkException(vegas(snpsGSC[[1]], 12, geneSetAnalysis[["ldMat"]]))
	checkException(vegas(snpsGSC[[1]], TRUE, geneSetAnalysis[["ldMat"]]))
	checkException(vegas(snpsGSC[[1]], GRanges(c(0,0)), geneSetAnalysis[["ldMat"]]))
	
	checkException(vegas(snpsGSC[[1]], arrayDataGR, 1))
	checkException(vegas(snpsGSC[[1]], arrayDataGR, NA))
	checkException(vegas(snpsGSC[[1]], arrayDataGR, TRUE))
	checkException(vegas(snpsGSC[[1]], arrayDataGR, "yes"))	
		
	checkException(vegas(snpsGSC[[1]], arrayDataGR, geneSetAnalysis[["ldMat"]], 
	    correction=NA))
	checkException(vegas(snpsGSC[[1]], arrayDataGR, geneSetAnalysis[["ldMat"]], 
	    correction="yes"))
	checkException(vegas(snpsGSC[[1]], arrayDataGR, geneSetAnalysis[["ldMat"]], 
	    correction=12))
	
	checkException(vegas(snpsGSC[[1]], arrayDataGR, geneSetAnalysis[["ldMat"]], 
	    seed=NA))
	checkException(vegas(snpsGSC[[1]], arrayDataGR, geneSetAnalysis[["ldMat"]], 
	    seed="yes"))
	checkException(vegas(snpsGSC[[1]], arrayDataGR, geneSetAnalysis[["ldMat"]], 
	    seed=TRUE))

	checkException(vegas(snpsGSC[[1]], arrayDataGR, geneSetAnalysis[["ldMat"]], 
	    verbose=NA))
	checkException(vegas(snpsGSC[[1]], arrayDataGR, geneSetAnalysis[["ldMat"]], 
	    verbose=12))
	checkException(vegas(snpsGSC[[1]], arrayDataGR, geneSetAnalysis[["ldMat"]], 
	    verbose="yes"))
}
	
	
	

##' Calculates the p-value representing the association of the set
#' with the phenotype of interest.
##'
##' This function calculates a p-value for sets of SNPs that 
#' reside within a gene set collection. We calculate the null 
#' distribution by taking into account the observed correlation 
#' among the SNPs and simulating a specified number of statistics
#' from which the resulting p-value is calculated.
##' @title Calculate the P-Value for a Set Using the VEGAS Method
##' @param set A \code{GeneSet} object containing a set of genes 
#' of interest or a \code{GeneSetCollection} object containing a 
#' collection of GeneSets.
##' @param assoc_table An object of class \code{GRanges}. This 
#' object should at least contain columns SNP and P which hold SNP
#' rsIDs and their corresponding association test p-values, 
#' respectively.
##' @param ldMatrix A square, symmetric matrix of LD values, with
#' each row and column corresponding to each of the items in the 
#' set. The diagonal entries should be 1, indicating the LD 
#' between an item in the set and itself is 1.
##' @param num_sims A positive integer value for the number of
#' simulations to be performed. Default is 1000.
##' @param correction A logical argument indicating whether a 
#' value of one should be added to the numerator when calculating 
#' the p-value based upon the simulated statistics. By default, 
#' the correction is added and this argument is TRUE.
##' @param seed An integer argument indicating what the random
#' seed should be set to. This allows for replication of results.
#' The default is NULL, and a random seed will be set internally. 
##' @param verbose A logical argument indicating whether output 
#' should be printed. The default is FALSE.
##' @return An object with the corresponding VEGAS results. If set
#' is a \code{GeneSetCollection} (i.e., multiple sets of 
#' interest), then the corresponding \code{VEGASResultCollection}
#' is returned. If set is a \code{GeneSet}, a \code{VEGASResult} 
#' object will be returned.
##' @references Liu, Jimmy Z. and Mcrae, Allan F. et al. 
#' A Versatile Gene-Based Test for Genome-Wide Association 
#' Studies.
#' The American Journal of Human Genetics, 2010.
##' @author Caitlin McHugh \email{mchughc@@uw.edu}
##' @export
vegas <- function(set, assoc_table, ldMatrix, num_sims=1000, 
    correction=TRUE, seed=NULL, verbose=FALSE)
{
    if(!is.logical(verbose)) stop("verbose must be either TRUE 
        or FALSE.")
    if(!is.logical(correction)) stop("correction must be either 
        TRUE or FALSE.")
    if(!all.equal(num_sims,as.integer(num_sims))) stop("num_sims 
        must be an integer.")  
    if(num_sims<=0){stop("num_sims must be positive.")}  
    
    if(num_sims<100 & verbose){
    	print("Results from running less than 100 simulations should be
            interpreted carefully.")
    }
    
    if(!(is.null(seed)|is.numeric(seed))) stop("seed must be a numeric value.")
    
    if(!is(ldMatrix, "data.frame")){
    	ldMatrix <- as.matrix(ldMatrix)
    	message("Warning: coercing ldMatrix from data.frame to matrix.")
    }
    
    # first, remove SNPs with all NA values in the LD matrix
    naSNPS <- apply(ldMatrix,1,function(x){sum(is.na(x))})
    torm <- names(naSNPS)[naSNPS>(nrow(ldMatrix)-3)]
    if(length(torm)>0){
	    colsKp <- !is.element(rownames(ldMatrix),torm)
    	ldMatrix <- ldMatrix[colsKp,colsKp]
    }
    
    # then remove SNPs with just one NA value in the LD matrix
    naSNPS <- apply(ldMatrix,1,function(x){sum(is.na(x))})
    torm <- names(naSNPS)[naSNPS>0]
    colsKp <- !is.element(rownames(ldMatrix),torm)
    ldMatrix <- ldMatrix[colsKp,colsKp]

    if(nrow(ldMatrix)!=ncol(ldMatrix)) stop("ldMatrix needs to 
        be square.")
    if(!isSymmetric(ldMatrix,tol=1e-04)) stop("ldMatrix needs to 
        be symmetric.")

    di <- diag(ldMatrix)
    if(!all(round(di,digits=5)==1)) stop("ldMatrix should have 
        all 1's on the diagonal.")
    
    if(is(set)[1]=="GeneSetCollection"){
    # loop through the gene sets, get the analysis results
        res <- bplapply(set,
                   function(set, assoc_table, ldMatrix, num_sims, correction,
                            seed, verbose) 
                   {
                       library(cpvSNP)
                       vegasPrep(set, assoc_table, ldMatrix, num_sims,
                                 correction, seed, verbose)
                   }, assoc_table=assoc_table, ldMatrix=ldMatrix,
                   num_sims=num_sims, correction=correction, seed=seed,
                   verbose=verbose)
	    return(new("VEGASResultCollection", res))
    }else{
    	res <- vegasPrep(set,assoc_table=assoc_table,
    	    ldMatrix=ldMatrix,num_sims=num_sims,
            correction=correction,seed=seed,verbose=verbose)
    	return(res)
    }
}

setMethod("geneSetName", "GLOSSIResult",
function(object){object@geneSetName}
)
setMethod("geneSetName", "GLOSSIResultCollection",
function(object){lapply(object, function(y) y@geneSetName)}
)

setMethod("pValue", "GLOSSIResult",
function(object){object@pValue}
)
setMethod("pValue", "GLOSSIResultCollection",
function(object){lapply(object, function(y) y@pValue)}
)

setMethod("degreesOfFreedom", "GLOSSIResult",
function(object){object@degreesOfFreedom}
)
setMethod("degreesOfFreedom", "GLOSSIResultCollection",
function(object){lapply(object, function(y) y@degreesOfFreedom)}
)

setMethod("statistic", "GLOSSIResult",
function(object){object@statistic}
)
setMethod("statistic", "GLOSSIResultCollection",
function(object){lapply(object, function(y) y@statistic)}
)

##' Calculates the p-value representing the association of the 
#' set with the phenotype of interest, assuming all items in the
#' set are independent, using the GLOSSI method.
##'
##' This function calculates a p-value for sets of SNPs that
#' reside within a gene set collection. We calculate the chi-
#' square p-values and statistic by applying Fisher's 
#' transformation to the observed p-values. 
##' @title Calculate a Chi-squared Statistic and P-Value for
#' Independent Items of a Set Using the GLOSSI Method
##' @param pvals A numerical vector of p-values with names 
#' corresponding to elements listed in the \code{geneIds} slot in 
#' the snp.gsc object.
##' @param snp.gsc An object of class \code{GeneSetCollection} 
#' where geneIds holds the items of each set corresponding to the 
#' pvals.
##' @return An object with the corresponding GLOSSI results. If 
#' snp.gsc is a \code{GeneSetCollection} (i.e., multiple sets of 
#' interest), then the corresponding \code{GLOSSIResultCollection}
#' is returned. If snp.gsc is a \code{GeneSet}, a \code{GLOSSIResult}
#' object will be returned.
##' @references Chai, High-Seng and Sicotte, Hughes et al. 
#' GLOSSI: a method to assess the association of genetic loci-sets 
#' with complex diseases.
#' BMC Bioinformatics, 2009.
##' @author Jason Hackney, Jessica Larson, Caitlin McHugh 
#' \email{mchughc@@uw.edu}
##' @export
glossi <- function(pvals, snp.gsc)
{
    if(is.null(names(pvals))) stop("Names of p-values must 
        correspond to SNPs.")
    
    # subset the gene stats to only include genes in the gene set    
    pvals <- pvals[!is.na(pvals)]
    pathway.snps <- unique(unlist(geneIds(snp.gsc)))
    common.snps <- intersect(names(pvals), pathway.snps)
    pvals <- pvals[common.snps]
    set.names <- names(snp.gsc)
    
    if(is(snp.gsc)[1]=="GeneSetCollection"){
    	res <- sapply( set.names, function(x){glossiMarginal(pvals,
            snp.gsc[[x]])} )
    	return( new("GLOSSIResultCollection", res) )
    }
    
    if(is(snp.gsc)[1]=="GeneSet"){
    	res <- glossiMarginal(pvals,snp.gsc)
    	return(res)
    }
    
}

## show
setMethod("show", "VEGASResult",
function(object){
    cat(paste0( "VEGAS results for ", object@geneSetName, "\n"))
    cat(paste0("p-value = ", signif(object@pValue,3), "\n") )
    cat(paste0("observed statistic = ", 
        signif(object@statistic,3), "\n") )
    cat(paste0("mean of simulated statistics = ", 
        signif(mean(object@simulatedStats),3), "\n"))
    cat(paste0("variance of simulated statistics = ", 
        signif(var(object@simulatedStats),3), "\n"))
}
)

setMethod("show", "GLOSSIResult",
function(object){
    cat(paste0( "GLOSSI results for ", object@geneSetName, "\n"))
    cat(paste0("p-value = ", signif(object@pValue,3), "\n"))
    cat(paste0("observed statistic = ", 
        signif(object@statistic,3), "\n"))
    cat(paste0("degrees of freedom = ", object@degreesOfFreedom), "\n")
}
)

setMethod("show", "GeneSetResult",
function(object){
    cat(paste0( "Gene Set results for ", object@geneSetName, "\n"))
    cat(paste0("p-value = ", signif(object@pValue,3), "\n"))
    cat(paste0("observed statistic = ", 
        signif(object@statistic,3), "\n"))
    cat(paste0("degrees of freedom = ", object@degreesOfFreedom), "\n")
}
)

##' Plots association test p-values, highlighting p-values within
#'  gene sets.
##'
##' Creates a density plot of all -log10(p-value), and overlaying 
#' a density plot of -log10(p-value) for SNPs within a gene set 
#' of interest.
##' @title Create a Density Plot of P-Values, Highlighting SNPs 
#' Within a Gene Set
##' @param pvals A numerical vector of p-values with names 
#' corresponding to elements listed in the geneIds slot in the 
#' set object.
##' @param set An object of class \code{GeneSet} where geneIds holds 
#' the items of each set corresponding to the pvals.
##' @param title A character string containing the title of the plot.
#' Default is "", a blank title.
##' @param geneSetColor A character vector holding the color the 
#' highlighted SNPs in the set object should be. Default is 
#' \code{red}.
##' @param xlab A character vector with the label for the x-axis on
#'  the plot. Default is \code{-log10(PValue)}.
##' @param ylab A character vector holding the label for the y-axis 
#' on the plot. Default is \code{Density}.
##' @param xlim A vector of two numeric values with the limits of 
#' plotting on the x-axis. By default, the range of the x-axis 
#' values will be determined from the data.
##' @param ylim A vector with two numeric values used as limits for 
#' plotting on the y-axis. By default, the range of the y-axis values
#'  will be determined from the data.
##' @param ... Further arguments to be passed to the plotting methods,
#' such as graphical parameters.
##' @return Creates a plot.
##' @author Jason Hackney, Jessica Larson, Caitlin McHugh
#' \email{mchughc@@uw.edu}
##' @export
assocPvalBySetPlot <- function(pvals, set, title="", 
    geneSetColor = "red", 
    xlab = expression(-log[10]~p-value), 
    ylab = "Density",
    xlim=NULL, ylim=NULL, ...)
{
    if(!is(set)[1]=="GeneSet"){
        stop("set must be of class GeneSet.") }
	
    geneSetColor <- rgb(t(col2rgb(geneSetColor)), alpha=90, 
        maxColorValue=255)
        
	setlocs <- names(pvals) %in% geneIds(set)
    setStats <- pvals[setlocs]	
	
    if(length(setlocs)<1){
        stop("No pvalues included for ids listed in geneIds(set).") }
	
    data <- list("main" = -log10(pvals+1e-09), 
        "geneSet" = -log10(setStats+1e-09))
    ## data is a list of numerical vectors: 'main', 'geneSet'
    dens <- bplapply(data , function(x) { if( length( x ) > 1 ){
        density( x, na.rm = TRUE )
	} else { NULL } } )
    if(is.null(xlim)) {
        xlim <- range(sapply(dens, function(x) {
	    if(!is.null(x)) { range(x$x, na.rm = TRUE) }
	    else { return(NA) }
            }), na.rm =TRUE)
    }
    if ( is.null(ylim) ) {
        ylim <- range(sapply(dens, function(x) {
	    if(!is.null(x)) {
	    range(x$y, na.rm = TRUE)
            } else { return(NA) }
	    }), na.rm =TRUE)
    }
    xlim <- c(0, max(abs(xlim)))
    for(n in setdiff(c("main", "geneSet"), names(dens))){ 
        dens[n]= NA }

    plot(dens$main, xlim = xlim, ylim = ylim, xlab = xlab, 
        main=title, ylab = ylab, col = "grey75", xaxt='n', ...)
	legend("topright", c("all assoc results","Set of interest"),
        col=c("grey75","red"), lty=c(1,1), lwd=c(3,3))

    polygon(dens$main, col="grey75", border="grey75")

    lines(dens$geneSet, xlim = xlim, ylim = ylim, xlab = xlab, 
        ylab = ylab, col = geneSetColor, xaxt='n', lwd=3)
    points(data$geneSet, y=jitter(rep(0,length(data$geneSet)), factor=0.1),
        cex=1, pch=19, col=rgb(t(col2rgb(geneSetColor)), alpha=100,
            maxColorValue=255))
}

##' Creates a GRanges object used for SNP set analysis.
##'
##' This function takes a data.frame and creates a GRanges 
#' object used for SNP set analysis.
##' @title Create a GRanges Object 
##' @param arrayData A data.frame object containing array data from 
#' which the \code{GRanges} object will be created. This object is expected
#' to include the position and chromosome, along with a SNP 
#' identifier and corresponding p-value.
##' @param positionName The name of the column in the filepathData 
#' object that holds position data for each probe. By default, this 
#' value is NULL and there are columns Start and End which hold 
#' this information.
##' @param chromosomeName The name of the column in the filepathData 
#' object that holds chromosome data for each probe. By default, 
#' this value is \code{chromosome}.
##' @param chromosomeNameConvention The naming convention used for the 
#' chromosomes, either \code{NCBI} (default), \code{UCSC}, or a user
#' specified mapping. By default, this is \code{NCBI}, indicating the autosomes
#' are stored as integers and the mitochondrial and sex chromosomes are stored
#' as \code{MT} and \code{X, Y}, respectively. The \code{UCSC} naming 
#' convention appends "chr" to each chromosome, and the mitochondrial coding is
#' "chrM." Alternatively, a user can supply the mapping from each chromosome
#' to the UCSC naming convention in the form of a character vector where the
#' elements are the UCSC names and the corresponding names of the elements are
#' the current chromosome names. An `NA' chromosome will be mapped to unknown.
#' For more information, please see the \code{GenomeInfoDb} BioConductor R package.
##' @param verbose A logical argument indicating whether output 
#' should be printed. The default is FALSE.
##' @return A GRanges object.
##' @author Jason Hackney, Jessica Larson, Caitlin McHugh
#' \email{mchughc@@uw.edu}
##' @export
createArrayData <- function(arrayData, 
    positionName=NULL, chromosomeName="chromosome", 
    chromosomeNameConvention="NCBI", verbose=TRUE){

    if(!is(arrayData, "data.frame")){ 
        stop("arrayData must be a data.frame object.")
    }

    names(arrayData)[is.element(names(arrayData),
        chromosomeName)] <- "chromosome"
    if(is.null(arrayData$chromosome)){
        stop("Data must have a `chromosome' variable.")    	
    }

    if(!is.element(chromosomeNameConvention, c("NCBI", "UCSC"))){
        # check to be sure its a vector with old/new names mapped
        chrs <- unique(arrayData$chromosome)
        if(!all(is.element(names(chromosomeNameConvention)),chrs)){
            stop("There is an error in the chromosomeNameConvention argument.")
        }
        if(length(chromosomeNameConvention)!=length(chrs)){
        	stop("The chromosomeNameConvention argument does not include all
        	    chromosomes listed in the data.")
        }
    }   

    if(!is.null(arrayData$FILTER)){
        arrayData$FILTER <- factor(arrayData$FILTER)
    }
    if(!is.null(arrayData$REF)){arrayData$REF <- factor(arrayData$REF)}
    if(!is.null(arrayData$ALT)){arrayData$ALT <- factor(arrayData$ALT)}
    if(is.null(positionName)&(is.null(arrayData$Start)|
        is.null(arrayData$End))){
        stop("Data must have a `position' variable.")
    }
    if(!is.null(positionName)){
        thisCol <- which(names(arrayData)==positionName)
        arrayData$Start <- arrayData[,thisCol]
        arrayData$End <- arrayData[,thisCol] 
    }
            
    # subset to only those that have calculated P-values
    arrayData <- arrayData[!is.na(arrayData$P),]
            
    arrayData$Start[is.na(arrayData$chromosome)] <- 1
    arrayData$End[is.na(arrayData$chromosome)] <- 1
                
    arrayData$chromosome[is.na(arrayData$chromosome)] <- 'U'
    
    if(chromosomeNameConvention=="NCBI"){
        arrayData$chromosome[is.element(arrayData$chromosome,0)] <- 'U'
        arrayData$chromosome[is.element(arrayData$chromosome,27)] <- 'U'
        arrayData$chromosome[is.element(arrayData$chromosome,"MT")] <- 'M'    
           
        arrayData$chromosome <- factor(arrayData$chromosome)
        levels(arrayData$chromosome) <- paste('chr',
            levels(arrayData$chromosome), sep = '')
    }
    
    if(!is.element(chromosomeNameConvention,c("NCBI","UCSC"))){
    	chrFact <- as.factor(arrayData$chromosome)
    	arrayData$chromosome <- chromosomeNameConvention[chrFact]
    }
        
    arrayData$chromosome <- factor(arrayData$chromosome,
        levels=c("chr1", "chr2", "chr3", "chr4", "chr5", "chr6", "chr7", 
        "chr8", "chr9", "chr10", "chr11", "chr12", "chr13", "chr14", 
        "chr15", "chr16", "chr17", "chr18", "chr19", "chr20", "chr21", 
        "chr22", "chrX", "chrY", "chrM", "chrXY", "chrU"))
                
    if(verbose){ message("making GRanges object") }
     
    arrayData <- arrayData[order(arrayData$chromosome, arrayData$Start),] 
            
    array.ranges <- GRanges( Rle(arrayData$chromosome), 
        IRanges(start=arrayData$Start, end=arrayData$End) )
    elementMetadata(array.ranges) <- arrayData

    return(array.ranges)
}

setMethod("geneSetName", "GeneSetResult",
function(object){object@geneSetName}
)
setMethod("geneSetName", "GeneSetResultCollection",
function(object){lapply(object, function(y) y@geneSetName)}
)

setMethod("pValue", "GeneSetResult",
function(object){object@pValue}
)
setMethod("pValue", "GeneSetResultCollection",
function(object){lapply(object, function(y) y@pValue)}
)

setMethod("degreesOfFreedom", "GeneSetResult",
function(object){object@degreesOfFreedom}
)
setMethod("degreesOfFreedom", "GeneSetResultCollection",
function(object){lapply(object, function(y) y@degreesOfFreedom)}
)

setMethod("statistic", "GeneSetResult",
function(object){object@statistic}
)
setMethod("statistic", "GeneSetResultCollection",
function(object){lapply(object, function(y) y@statistic)}
)

setClass("GeneSetResult",
    representation(
        geneSetName = "character",
        pValue = "numeric",
        degreesOfFreedom = "integer",
        statistic = "numeric")
)
setClass("GeneSetResultCollection",
    contains="list",
    validity = function(object) {
        msg <- NULL
        if ( !all(sapply(object, is, "GeneSetResult")) )
            msg <- c(msg, "members must all be 'GeneSetResult' classes")
        if ( !is.null(msg) )
            msg
        else
            TRUE
    }
)

setClass("VEGASResult",
	contains="GeneSetResult",
    representation( 
        simulatedStats = "vector")
)

setClass("VEGASResultCollection",
    contains="list",
    validity = function(object) {
        msg <- NULL
        if ( !all(sapply(object, is, "VEGASResult")) )
            msg <- c(msg, "members must all be 'VEGASResult' classes")
        if ( !is.null(msg) )
            msg
        else
            TRUE
    }
)

setClass("GLOSSIResult",
    contains="GeneSetResult"
)

setClass("GLOSSIResultCollection",
    contains="list",
    validity = function(object) {
        msg <- NULL
        if ( !all(sapply(object, is, "GLOSSIResult")) )
            msg <- c(msg, "members must all be 'GLOSSIResult' classes")
        if ( !is.null(msg) )
            msg
        else
            TRUE
    }
)

##' Plots association test p-values against number of SNPs per
#' gene set.
##'
##' Creates a plot of all p-values against the number of
#' SNPs in the gene set, for all sets within a collection.
##' @title Create a Plot of P-Values Against Number of SNPs
#' per Set for All Sets in a Collection
##' @param set An object of class \code{GeneSetResultCollection}. 
##' @param title A character string containing the title of the plot.
#' Default is "", a blank title.
##' @param xlab A character vector with the label for the x-axis on
#'  the plot. Default is \code{SNPs per Gene Set}.
##' @param ylab A character vector holding the label for the y-axis 
#' on the plot. Default is \code{PValue}.
##' @param ... Further arguments to be passed to the plotting methods,
#' such as graphical parameters.
##' @return Creates a plot.
##' @author Caitlin McHugh \email{mchughc@@uw.edu}
##' @export
plotPvals <- function(set, title="", xlab = "SNPs per Gene Set", 
ylab = "PValue", ...)
{
	if(!is.element(class(set),c("GeneSetResultCollection",
	    "GLOSSIResultCollection","VEGASResultCollection"))){
	        stop("set must be a GeneSetResultCollection object.")}

	result <- data.frame(df=unlist(degreesOfFreedom(set)),
	    pval=unlist(pValue(set)))
    ggplot(result, aes(x=df,y=pval, ...)) +
    geom_point() + xlab(xlab) + ylab(ylab) + ggtitle(title)
}

##' Calculates the p-value representing the association of the 
#' set with the phenotype of interest.
##'
##' This is a helper function to calculate the p-value for a set 
#' of SNPs that reside within a gene set collection. The 
#' correlation among the SNPs is taken into account by the LD 
#' matrix. The resulting p-value is calculated from a null 
#' distribution that is simulated \code{num_sims} times based upon
#' the specified correlation structure.
##' @title Calculate the P-Value for a Set Using the VEGAS Method
##' @param pvals A vector of p-values corresponding to items in 
#' the set.
##' @param ld_matrix A square, symmetric matrix of LD values, with
#' each row and column corresponding to each of the items in the 
#' set. The diagonal entries should be 1, indicating the LD 
#' between an item in the set and itself is 1.
##' @param num_sims An integer value for the number of simulations
#' to be performed. 
##' @param correction A logical argument indicating whether a 
#' value of one should be added to the numerator when calculating 
#' the p-value based upon the simulated statistics. By default, 
#' the correction is added. An argument of FALSE will not add one
#' to the numerator.
##' @param seed An integer argument indicating what the random
#' seed should be set to. This allows for replication of results.
#' The default is NULL, and a random seed will be set internally. 
##' @param verbose A logical argument indicating whether periodic
#' output should be printed. Defaults to FALSE, indicating no 
#' output will be printed.
##' @return A \code{VEGASResult} object with the corresponding 
#' VEGAS results.
##' @author Caitlin McHugh \email{mchughc@@uw.edu}
##' @export
vegasMarginal <- function(pvals, ld_matrix, num_sims, 
    correction=TRUE, seed=NULL, verbose=FALSE)
{
    if(!length(pvals)==nrow(ld_matrix)){
        stop("pvals should correspond to the entries 
            in ldMatrix.")
    }
    if(nrow(ld_matrix)==0) stop("no SNPs in this gene set.")
    
    if(is.null(seed)) seed <- rbinom(1,1e6,0.5)
	set.seed(seed)
	
    ld_matrix <- make.positive.definite(ld_matrix)

    # take transpose of the the cholesky decomp of the ld_matrix
    chol_matrix <- t(chol(ld_matrix))
    
    # calculate the observed sum of chisqs for this gene set
    chisqVals <- qchisq(pvals,df=1,ncp=0,lower.tail=FALSE)
    chisq_obs <- sum(chisqVals)
    
    # simulate num_sims chisq stats with the same correlation
    chisq_sim <- simulate_chisq(chol_matrix,num_sims)
    
    output <- vector("list",3)
    names(output) <- c("pvalue","statistic","simulated.stats")
    numerP <- sum(chisq_sim>chisq_obs)
	if(correction){
        output[["pvalue"]] <- (numerP+1)/(num_sims+1)
	}else{
        output[["pvalue"]] <- numerP/num_sims
	}
    output[["statistic"]] <- chisq_obs
    output[["simulated.stats"]] <- chisq_sim
    
    return(output)
}

setMethod("geneSetName", "VEGASResult",
function(object){object@geneSetName}
)
setMethod("geneSetName", "VEGASResultCollection",
function(object){lapply(object, function(y) y@geneSetName)}
)

setMethod("pValue", "VEGASResult",
function(object){object@pValue}
)
setMethod("pValue", "VEGASResultCollection",
function(object){lapply(object, function(y) y@pValue)}
)

setMethod("degreesOfFreedom","VEGASResult",
function(object){object@degreesOfFreedom}
)
setMethod("degreesOfFreedom", "VEGASResultCollection",
function(object){lapply(object,function(y) y@degreesOfFreedom)}
)

setMethod("statistic", "VEGASResult",
function(object){object@statistic}
)
setMethod("statistic", "VEGASResultCollection",
function(object){lapply(object, function(y) y@statistic)}
)

setMethod("simulatedStats", "VEGASResult",
function(object){object@simulatedStats}
)
setMethod("simulatedStats", "VEGASResultCollection",
function(object){lapply(object, function(y) y@simulatedStats)}
)

##' A helper function to calculate the chi-squared statistic 
#' corresponding to an observed set of p-values.
##'
##' This function calculates a chi-squared statistic from a set of
#' pvalues.
##' @title Calculate a Chi-squared Statistic and P-Value for Items
#' in a Set
##' @param pval A vector of p-values of length equal to the number
#' of geneIds in the \code{GeneSetCollection} object.
##' @param set An element of a \code{GeneSetCollection} object.
##' @return An object of type \code{GLOSSIResult}.
##' @author Jason Hackney, Jessica Larson
##' @export
glossiMarginal <- function(pval, set)
{
    ids <- intersect(unlist(geneIds(set)), names(pval))
    
    # need to add a bit so we don't have log(0)
    gamma.stat <- sum(-log(pval[ids]+.Machine$double.eps) )
    
    # find the corresponding gamma/chisq density pvalue
    gamma.p <- pgamma(gamma.stat, length(ids), lower.tail=FALSE)
    
    return( new("GLOSSIResult",
        geneSetName = setName(set),
        pValue = gamma.p,
        degreesOfFreedom = length(ids),
        statistic = gamma.stat) )
}

##' Simulates a value from a chi-squared distribution with a 
#' specified correlation structure.
##'
##' This function simulates a value from a chi-squared 
#' distribution with a specified correlation structure and degrees
#' of freedom corresponding to the number of rows in the 
#' correlation matrix.
##' @title Simulate a Chi-squared Statistic from a Distribution 
#' with a Specified Correlation Structure
##' @param corr_matrix A square matrix of correlations.
##' @param num_sims An integer value for the number of simulations
#' to be performed. 
##' @return A numeric value corresponding to a chi-squared 
#' statistic simulated from a chi-squared-like distribution with 
#' specified correlation structure and degrees of freedom 
#' corresponding to the number of rows in the correlation matrix.
##' @author Caitlin McHugh \email{mchughc@@uw.edu}
##' @export
simulate_chisq <- function(corr_matrix, num_sims)
{
    # sample n random standard normal values
    z <- rnorm(nrow(corr_matrix)*num_sims,mean=0,sd=1)
    zmat <- matrix(z,nrow=num_sims)
    # multiply them by the correlation structure
    z_corr <- zmat%*%corr_matrix
    
    # square and sum them
    zsq <- z_corr**2
    z_chisq <- rowSums(zsq)
    
    return(z_chisq)
}

setGeneric("geneSetName", function(object) standardGeneric("geneSetName") )

setGeneric("pValue", function(object) standardGeneric("pValue") )

setGeneric("simulatedStats", function(object) standardGeneric("simulatedStats") )

setGeneric("degreesOfFreedom", function(object) standardGeneric("degreesOfFreedom") )

setGeneric("statistic", function(object) standardGeneric("statistic") )

vegasPrep <- function(set, assoc_table, ldMatrix, num_sims,
    correction, seed, verbose){
	
    if(is(set)[1]=="GeneSetCollection"){gsn <- names(set)}
    if(is(set)[1]=="GeneSet"){gsn <- setName(set)}

    if(verbose){ message(paste("now analyzing gene set",gsn)) }
    snpsKp <- is.element(rownames(ldMatrix),
        unlist(geneIds(set)))&is.element(rownames(ldMatrix),
        assoc_table$SNP)
    this_ld <- ldMatrix[snpsKp,snpsKp]
	
    assoc_thisSet <- assoc_table[is.element(assoc_table$SNP,
        rownames(this_ld)),]
    this_vegas_result <- vegasMarginal(assoc_thisSet$P,this_ld,
        num_sims,correction,seed,verbose)
    if(is.null(this_vegas_result)){ return(NULL) }
	
    return( new("VEGASResult",
        geneSetName = gsn,
	    pValue = this_vegas_result[["pvalue"]],
	    degreesOfFreedom = dim(elementMetadata(assoc_thisSet))[1],
	    statistic = this_vegas_result[["statistic"]],
	    simulatedStats = this_vegas_result[["simulated.stats"]]) )
}

%\VignetteIndexEntry{Running gene set analyses with the "cpvSNP" package}
%\VignettePackage{cpvSNP}
\documentclass[12pt]{article}


<<style, eval=TRUE, echo=FALSE, results=tex>>=
BiocStyle::latex()
@


\usepackage{cite}
\usepackage{amsmath}
\usepackage{Sweave}
\SweaveOpts{keep.source=TRUE,eps=FALSE,pdf=FALSE,png=TRUE,include=FALSE,width=4,height=4.5,resolution=150}
\setkeys{Gin}{width=0.5\textwidth}

% use a vertical rule for R output instead of prompts for R input
\usepackage{fancyvrb}
\definecolor{darkgray}{gray}{0.2}
\DefineVerbatimEnvironment{Sinput}{Verbatim}{xleftmargin=1em,formatcom={\color{darkgray}}}
\DefineVerbatimEnvironment{Soutput}{Verbatim}{xleftmargin=1em,frame=leftline,framerule=.6pt,rulecolor=\color{darkgray},framesep=1em,formatcom={\color{darkgray}}}
\fvset{listparameters={\setlength{\topsep}{0pt}}}
\renewenvironment{Schunk}{\vspace{\topsep}}{\vspace{\topsep}}


\author{Caitlin McHugh$^{1,2*}$, Jason Hackney$^1$, and Jessica L. Larson$^1$ \\
	[1em] \small{$^{1}$ Department of Bioinformatics and Computational Biology, Genentech, Inc.} \\ 
	\small{$^{2}$ Department of Biostatistics, University of Washington} \\ 
	\small{\texttt{$^*$mchughc (at) uw.edu}}}

\title{Combining SNP P-Values in Gene Sets: the cpvSNP Package}

\date{\today}


\begin{document}

\maketitle
\tableofcontents
\newpage


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\section{Introduction }
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Genome-wide association studies (GWAS) have lead to the discovery of many disease-associated single 
nucleotide polymorphisms (SNPs).  Researchers are often interested in extending these studies to 
determine the genetic association of a given pathway (i.e., a gene set) 
with a certain phenotype.  Gene set methods allow users to combine SNP-level association $p$-values 
across multiple biologically related genes.

The \verb@cpvSNP@ package provides code for two gene set analysis methods [1-2] and 
accurately corrects for the correlation structure among observed SNPs. 
Both of the implemented methods translate a set of gene 
ids to their corresponding SNPs, and combine the $p$-values for those SNPs. 
Calculated statistics, degrees of freedom, and corresponding $p$-values are stored for each gene set.

This vignette describes a typical analysis workflow and includes some details regarding the statistical theory 
behind \verb@cpvSNP@.  For more technical details, please see references [1] and [2].



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\section{Example workflow for cpvSNP}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\subsection{Preparing a dataset for analysis}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
For our example, we will use a set of simulated data, the \verb@geneSetAnalysis@ dataset from the \verb@cpvSNP@ package.  
We begin by loading relevant libraries, sub-setting the data,
and running \verb@createArrayData@ on this data set.  

<<loadLibrariesAndData>>=
library(cpvSNP)
data(geneSetAnalysis)
names(geneSetAnalysis)
@

The \verb@geneSetAnalysis@ list holds four elements, each of which we will need for this vignette. 
The first object, \verb@arrayData@, is a \verb@data.frame@ containing the $p$-values, SNP ids, 
genomic position, and chromosome of all the probes in our hypothetical GWAS. 
Our first step is to use the \verb@cpvSNP@ function \verb@createArrayData@ to convert this \verb@data.frame@ 
to a \verb@GRanges@ object.

<<createArrayData>>=
arrayDataGR <- createArrayData(geneSetAnalysis[["arrayData"]], 
    positionName="Position")
class(arrayDataGR)
@


The \verb@geneSetAnalysis@ list also contains a  \verb@GeneSetCollection@ with two sets of interest.  
We can find the Entrez ids by accessing the \verb@geneIds@ slot of the \verb@GeneSetCollection@.
<<geneSets>>=
geneSets <- geneSetAnalysis[["geneSets"]]
geneSets
length(geneSets)
head(geneIds(geneSets[[1]]))
details(geneSets[[1]])
head(geneIds(geneSets[[2]]))
@

Our next data formatting step is to convert the ids in our \verb@GeneSetCollection@ from Entrez gene ids to their corresponding SNP ids.  
In this example, our SNP positions are coded in the hg19 genome build. 
Please be careful when converting gene ids to SNPs, as mappings change between genome build updates.
The \verb@geneToSNPList@ function requires gene transcripts stored as a \verb@GRanges@ object, along with 
the \verb@GRanges@ object specific to our study.
For this example, we will use the gene transcripts stored in the database \verb@TxDb.Hsapiens.UCSC.hg19.knownGene@.

<<geneToSNPList>>=
library(TxDb.Hsapiens.UCSC.hg19.knownGene)

txdb <- TxDb.Hsapiens.UCSC.hg19.knownGene
genesHg19 <- genes(txdb)

snpsGSC <- geneToSNPList(geneSets, arrayDataGR, genesHg19)
class(snpsGSC)
@

Note that the \verb@geneToSNPList@ function has a \verb@quiet@ option defaulted to \verb@TRUE@, which 
suppresses all warnings that may arise when finding overlaps between the genes in our collection and our study SNPs.
The default is set to \verb@TRUE@ because there are often warnings that are usually not an issue. 
However, please be aware that valid warnings may also be suppressed
if the \verb@quiet@ option is set to \verb@TRUE@.

We now have the two input files required to run GLOSSI [1] and VEGAS [2]: a \verb@GRanges@ object for the SNPs in our GWAS, and 
a \verb@GeneSetCollection@ with SNP ids for each gene in each set.

<<readyToRun>>=
arrayDataGR
snpsGSC
@



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\subsection{Running GLOSSI}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
An assumption of GLOSSI [1] is that our SNPs (and thus $p$-values) are independent.  In order to run \verb@glossi@, 
we must subset our \verb@arrayDataGR@ $p$-values to those from independent SNPs.

In the \verb@geneSetAnalysis@ list, we have included a 
vector of independent SNPs from our GWAS experiment. 
This list was created using a standard `LD-pruning' method from the PLINK software [3].

<<indep>>=
indep <- geneSetAnalysis[["indepSNPs"]]
head(indep)
dim(indep)
@

We now subset \verb@arrayDataGR@ to contain only independent SNPs, and create a new vector of 
$p$-values with names corresponding to these independent SNPs.
<<createIndepP>>=
pvals <- arrayDataGR$P[is.element(arrayDataGR$SNP, indep$V1)]
names(pvals) <- arrayDataGR$SNP[is.element(arrayDataGR$SNP, indep$V1)]
head(pvals)
@

We now have the proper input to call \verb@glossi@. 
We can consider all gene sets in our \verb@GeneSetCollection@, or call \verb@glossi@ on a just some of the sets.
Accessor functions for the resulting \verb@GLOSSIResultCollection@ allow us to view the results.

<<glossi>>=
gRes <- glossi(pvals, snpsGSC)
gRes

gRes2 <- glossi(pvals, snpsGSC[[1]])
gRes2

pValue(gRes)
degreesOfFreedom(gRes)
statistic(gRes)
@

Using the \verb@ReportingTools@ package, we can publish 
these results to a HTML page for exploration.  We first adjust for multiple testing.
<<publishMultiple, eval=FALSE >>=
pvals <- p.adjust( unlist( pValue(gRes) ), method= "BH" )
library(ReportingTools)
report <- HTMLReport (shortName = "cpvSNP_glossiResult",
        title = "GLOSSI Results", reportDirectory = "./reports")
publish(geneSets, report, annotation.db = "org.Hs.eg",
        setStats = unlist(statistic (gRes)), 
        setPValues = pvals)
finish(report)
@


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\subsection{Running VEGAS}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Unlike GLOSSI, which requires SNPs and $p$-values to be independent, VEGAS [2] accounts for correlation 
among SNPs and corresponding $p$-values.
We thus need a matrix of correlation values for each SNP in our GWAS. Most commonly, this correlation matrix 
holds linkage disequilibrium (LD) values.
Many R packages and online tools exist to calculate an LD matrix for observed raw data. 

Here, we briefly show how to calculate an LD matrix for chromosome 1 using the publicly available HapMap data, 
the R package \verb@snpStats@, and the PLINK software package [3].
This requires downloading PLINK file formatted data, extracting the probes on chromosome 1, and 
then calculating LD among SNPs in the \verb@snpsGSC@ elements.
<<hapmap, eval=FALSE>>=
download.file(
    url="http://hapmap.ncbi.nlm.nih.gov/genotypes/hapmap3_r3/plink_format/hapmap3_r3_b36_fwd.consensus.qc.poly.ped.gz",
    destfile="hapmap3_r3_b36_fwd.consensus.qc.poly.ped.gz")
download.file(
    url="http://hapmap.ncbi.nlm.nih.gov/genotypes/hapmap3_r3/plink_format/hapmap3_r3_b36_fwd.consensus.qc.poly.map.gz",
    destfile="hapmap3_r3_b36_fwd.consensus.qc.poly.map.gz")
system("gunzip hapmap3_r3_b36_fwd.consensus.qc.poly.ped.gz")
system("gunzip hapmap3_r3_b36_fwd.consensus.qc.poly.map.gz")
system("plink --file hapmap3_r3_b36_fwd.consensus.qc.poly --make-bed --chr 1")

genos <- read.plink(bed, bim, fam)
genos$genotypes
head(genos$map)
x <- genos[,is.element(genos$map$snp.name,c(geneIds(snpsGSC[[2]])))]
ldMat <- ld(x,y=x,stats="R.squared")
@

We have performed these steps already, and can simply use the LD matrix included in our \verb@geneSetAnalysis@ list, \verb@ldMat@ 
to call \verb@vegas@.  Note that the \verb@vegas@ method calculates simulated statistics (see Methods section below for more details).
<<vegas, eval=FALSE>>=
ldMat <- geneSetAnalysis[["ldMat"]]
vRes <- vegas(snpsGSC[1], arrayDataGR, ldMat)
vRes
summary(unlist(simulatedStats(vRes)))

pValue(vRes)
degreesOfFreedom(vRes)
statistic(vRes)
@


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\subsection{Visualizing Results}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
There are two plotting functions available in \verb@cpvSNP@ to visualize the results 
from the GLOSSI and VEGAS methods.

The \verb@plotPvals@ function plots the calculated $p$-values against the number of SNPs in each gene set, 
for each set in the original \verb@GeneSetCollection@and \verb@GLOSSIResultCollection@.
In this vignette we have only analyzed two gene sets, so this plot is not very informative.
The plot is included simply to demonstrate the plotting functions available in the \verb@cpvSNP@ package.
<<plotb, fig=TRUE>>=
plotPvals(gRes, main="GLOSSI P-values")
@

\begin{figure}
\centering
\includegraphics{cpvSNP-plotb}
\caption{The number of SNPs per gene set versus the $p$-value, for the GLOSSI methods.}
\end{figure}

The \verb@assocPvalBySetPlot@ function plots the GWAS $p$-values for each SNP in the original association study, 
as well as those for SNPs in a particular gene set.
This visualization enables an easy comparison of the $p$-values within a particular gene set to all $p$-values from our GWAS.  
Gene sets that are highly associated with the phenotype of interest will have a different distribution than all SNPs in our study.
<<plot2, fig=TRUE>>=
pvals <- arrayDataGR$P
names(pvals) <- arrayDataGR$SNP
assocPvalBySetPlot(pvals, snpsGSC[[2]])
@

\begin{figure}
\centering
\includegraphics{cpvSNP-plot2}
\caption{Density plots of all $p$-values, overlaid in red with $p$-values from the second gene set.}
\end{figure}



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\section{Methods in brief}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\subsection{GLOSSI methods}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
The GLOSSI [1] method assumes that our $p$-values are independently distributed. 
Define $J$ to be the total number of \textit{independent} SNPs for which we have association $p$-values, 
such that each locus $j$ has a corresponding $p$-value, $p_j$, $j \in \{1,\dots,J\}$. For this vignette, $J=302$.
Let $K$ be the total number of loci sets in which we are interested. For the example used in this vignette, $K=2$.

We begin by defining an indicator variable $g$ for each loci set $k$ and for each locus $j$, such that
$$
g_{jk}=\begin{cases} 1, &\mbox{if } j^{th} \mbox{ locus is in }k^{th} \mbox{ set }\\
0, & \mbox{otherwise} \end{cases}
$$

Note the sum of $g_{jk}$ is the size of loci-set $k$
\begin{equation*}
n_k = \sum_{j=1}^J g_{jk}
\end{equation*}

Our statistic for each loci-set $k$ is defined as
\begin{equation*}
S_{k_{obs}}= -2 \sum_{j=1}^J g_{jk} log(p_j)
\end{equation*}

We know from Fisher's transformation that if the $p_j \overset{iid}{\sim} Unif(0,1)$ then $S_{k_{obs}} \sim \chi_{2n_k}^2$. 
Thus, to calculate the corresponding $p$-value for loci-set $k$, we simply use the corresponding $\chi^2$ distribution for each set. 
Note the degrees of freedom in the null distribution takes into account the size of the loci-set, $n_k$.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\subsection{VEGAS methods}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
The VEGAS [2] method does not require independent SNPs, but rather a matrix of correlation values among the SNPs being considered. 
These correlation values can be correlation coefficients, a composite LD measure, or similar. 
We denote the correlation matrix for a particular loci-set $k$ as $\Sigma_k$, where each row and column corresponds to a SNP in $k$. 
This matrix must be square, symmetric, and have values of $1$ on the diagonal. 

To calculate a $p$-value for loci-set $k$ that takes into account the correlation structure, we begin by simulating a vector $z\sim N(0,1)$ with length $n_k$. 
We take the Cholesky decomposition of $\Sigma_k$ and multiply this by $z$ to define a Multivariate Normal random variable $z' \sim MVN(0,\Sigma_k)$. 
To define a statistic from this null distribution that now has the same correlation structure as our observed data, we calculate
\begin{equation*}
S_k = \sum_{i=1}^{n_k} [z_i chol(\Sigma_k)]^2
\end{equation*}
We simulate the vector $z$ a total of $n_{sims}$ times. 
We calculate the observed $p$-value as 
\begin{equation*}
\frac{\#(S_k>S_{k_{obs}})+1}{(n_{sims}+1)}.
\end{equation*}



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\section{Session Info}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
<<sessInfo, results=tex, echo=FALSE>>=
toLatex(sessionInfo())
@

<<resetOptions, echo=FALSE>>=
options(prompt="> ", continue="+ ")
@ 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\section{References}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
1. Chai, High-Seng and Sicotte, Hughes et al. 
GLOSSI: a method to assess the association of genetic loci-sets with complex diseases.
BMC Bioinformatics, 2009.

2. Liu, Jimmy Z. and Mcrae, Allan F. et al. 
A Versatile Gene-Based Test for Genome-Wide Association Studies.
The American Journal of Human Genetics, 2010.

3. Purcell S., Neale B., and Sham P.C. et al. 
        PLINK: a toolset for whole-genome association and population-based 
	linkage analysis. American Journal of Human Genetics, 2007.

\end{document}

\name{exampleArrayData}
\docType{data}
\alias{exampleArrayData}
\title{SNP Array Data to Use for Gene Set Analysis}
\description{
	This dataset provides simulated association p-values for an example set of SNP rsIDs, including their chromosome and position along the chromosome. The data in this file are all SNPs that lie on chromosome 1.
}
\usage{exampleArrayData}
\format{A data.frame containing 119,487 rows and the following 4 columns:
\tabular{ll}{
	P    \tab simulated p-values\cr
	SNP  \tab SNP rsIDs\cr
	Position \tab SNP position\cr
	chromosome \tab SNP chromosome
}
}
\keyword{datasets}

\name{VEGASResultCollection-class}
\Rdversion{1.1}
\docType{class}
\alias{VEGASResultCollection-class}

\title{Class \code{"VEGASResultCollection"}}
\description{
Objects of this class store results from running VEGAS. VEGASResultCollection objects contain a list of VEGASResult objects (one result for each GeneSet)
}
\section{Objects from the Class}{
Objects can be created by calls of \code{vegas}.
}
\section{Methods}{
No methods defined with class "VEGASResult" in the signature.
}
\section{Accessors}{
\describe{
In the code snippets below, \code{object} is a VEGASResultCollection object.
    \item{\code{geneSetName(object)}:}{A list of the names of the gene sets.}
    \item{\code{pValue(object)}:}{A list of numeric p-values.}
    \item{\code{degreesOfFreedom(object)}:}{List of integer values of the degrees of freedom used in the statistical tests performed on each gene set.}
    \item{\code{statistic(object)}:}{A list of numeric values corresponding to the statistics calculated from the tests performed on each gene set.}
}
}
\author{
Caitlin McHugh
}

\seealso{
\code{\linkS4class{VEGASResult}}, \code{\linkS4class{GLOSSIResultCollection}} 
}
\examples{
showClass("VEGASResultCollection")
}
\keyword{classes}

\name{GLOSSIResultCollection-class}
\Rdversion{1.1}
\docType{class}
\alias{GLOSSIResultCollection-class}

\title{Class \code{"GLOSSIResultCollection"}}
\description{
Objects of this class store results from running GLOSSI. GLOSSIResultCollection objects contain a list of GLOSSIResult objects (one result for each GeneSet)
}
\section{Objects from the Class}{
Objects can be created by calls of \code{glossi}.
}
\section{Methods}{
No methods defined with class "GLOSSIResult" in the signature.
}
\section{Accessors}{
\describe{
In the code snippets below, \code{object} is a GLOSSIResultCollection object.
    \item{\code{geneSetName(object)}:}{A list of the names of the gene sets.}
    \item{\code{pValue(object)}:}{A list of numeric p-values.}
    \item{\code{degreesOfFreedom(object)}:}{List of integer values of the degrees of freedom used in the statistical tests performed on each gene set.}
    \item{\code{statistic(object)}:}{A list of numeric values corresponding to the statistics calculated from the tests performed on each gene set.}
}
}
\author{
Caitlin McHugh
}

\seealso{
\code{\linkS4class{GLOSSIResult}}, \code{\linkS4class{VEGASResultCollection}} 
}
\examples{
showClass("GLOSSIResultCollection")
}
\keyword{classes}

\name{GeneSetResultCollection-class}
\Rdversion{1.1}
\docType{class}
\alias{GeneSetResultCollection-class}

\title{Class \code{"GeneSetResultCollection"}}
\description{
Objects of this class store results from running gene set methods such as GLOSSI or VEGAS. GeneSetResultCollection objects contain a list of GeneSetResult objects (one result for each GeneSet)
}
\section{Objects from the Class}{
Objects can be created by calls of \code{glossi} or \code{vegas}.
}
\section{Methods}{
No methods defined with class "GeneSetResultCollection" in the signature.
}
\section{Accessors}{
\describe{
In the code snippets below, \code{object} is a GeneSetResultCollection object.
    \item{\code{geneSetName(object)}:}{A list of the names of the gene sets.}
    \item{\code{pValue(object)}:}{A list of numeric p-values.}
    \item{\code{degreesOfFreedom(object)}:}{List of integer values of the degrees of freedom used in the statistical tests performed on each gene set.}
    \item{\code{statistic(object)}:}{A list of numeric values corresponding to the statistics calculated from the tests performed on each gene set.}
}
}
\author{
Caitlin McHugh
}

\seealso{
\code{\linkS4class{GeneSetResult}}, \code{\linkS4class{GLOSSIResultCollection}} 
}
\examples{
showClass("GeneSetResultCollection")
}
\keyword{classes}

\name{summary-methods}
\docType{methods}
\alias{show}
\alias{show-methods}
\alias{show,VEGASResult-method}
\alias{show,GLOSSIResult-method}
\title{ ~~ Methods for Function \code{show} in Package \pkg{base} ~~}
\description{
 ~~ Methods for function \code{show} in package \pkg{base} ~~
}
\section{Methods}{
\describe{
 Defined methods include:
	\item{
		\code{signature(x = "VEGASResult")},
		\code{signature(x = "GLOSSIResult")}
		}{
			These methods display the corresponding statistics and p-values for the corresponding VEGAS or GLOSSI analysis.
		}
	}
}
\author{
Caitlin McHugh
}
\keyword{methods}

\name{geneSets}
\docType{data}
\alias{geneSets}
\title{Gene Set Collection object}
\description{
	This gene set collection object provides four gene sets of varying sizes to be used as examples of how to run the \code{cpvSNP} package functions.
}
\usage{geneSets}
\format{A \code{GeneSetCollection} object of length 4. The values stored in the \code{geneIds} slot are Entrez ids. Gene sets 1-3 were randomly selected with sizes 100, 200 and 50, respectively. The fourth GeneSet holds genes in which the simulated p-values for corresponding SNPs are significant.}
\keyword{datasets}

\name{glossiMarginal}
\alias{glossiMarginal}
\title{Calculate a Chi-squared Statistic and P-Value for Items
in a Set}
\usage{
glossiMarginal(pval, set)
}
\arguments{
\item{pval}{A vector of p-values of length equal to the number
of geneIds in the \code{GeneSetCollection} object.}

\item{set}{An element of a \code{GeneSetCollection} object.}
}
\value{
An object of type \code{GLOSSIResult}.
}
\description{
A helper function to calculate the chi-squared statistic
corresponding to an observed set of p-values.
}
\details{
This function calculates a chi-squared statistic from a set of
pvalues.
}
\author{
Jason Hackney, Jessica Larson
}
\examples{
    set.seed(30)
    pvals <- runif(100)
    names(pvals) <- paste0("rs", 1:100)
    snpGS5 <- GeneSet(geneIds=names(pvals)[1:5], setName="set5")
    res <- glossiMarginal (pvals, snpGS5)
}

% Generated by roxygen2 (4.0.1): do not edit by hand
\name{assocPvalBySetPlot}
\alias{assocPvalBySetPlot}
\title{Create a Density Plot of P-Values, Highlighting SNPs
Within a Gene Set}
\usage{
assocPvalBySetPlot(pvals, set, title = "", geneSetColor = "red",
   xlab = expression(-log[10]~p-value), 
   ylab = "Density", 
   xlim = NULL, ylim = NULL,
  ...)
}
\arguments{
\item{pvals}{A numerical vector of p-values with names
corresponding to elements listed in the geneIds slot in the
set object.}

\item{set}{An object of class \code{GeneSet} where geneIds holds
the items of each set corresponding to the pvals.}

\item{title}{A character string containing the title of the plot.
Default is "", a blank title.}

\item{geneSetColor}{A character vector holding the color the
highlighted SNPs in the set object should be. Default is
\code{red}.}

\item{xlab}{A character vector with the label for the x-axis on
the plot. Default is \code{-log10(PValue)}.}

\item{ylab}{A character vector holding the label for the y-axis
on the plot. Default is \code{Density}.}

\item{xlim}{A vector of two numeric values with the limits of
plotting on the x-axis. By default, the range of the x-axis
values will be determined from the data.}

\item{ylim}{A vector with two numeric values used as limits for
plotting on the y-axis. By default, the range of the y-axis values
 will be determined from the data.}

\item{...}{Further arguments to be passed to the plotting methods,
such as graphical parameters.}
}
\value{
Creates a plot.
}
\description{
Plots association test p-values, highlighting p-values within
 gene sets.
}
\details{
Creates a density plot of all -log10(p-value), and overlaying
a density plot of -log10(p-value) for SNPs within a gene set
of interest.
}
\author{
Jason Hackney, Jessica Larson, Caitlin McHugh
\email{mchughc@uw.edu}
}

% Generated by roxygen2 (4.0.1): do not edit by hand
\name{vegasMarginal}
\alias{vegasMarginal}
\title{Calculate the P-Value for a Set Using the VEGAS Method}
\usage{
vegasMarginal(pvals, ld_matrix, num_sims, correction = TRUE, seed = NULL,
  verbose = FALSE)
}
\arguments{
\item{pvals}{A vector of p-values corresponding to items in
the set.}

\item{ld_matrix}{A square, symmetric matrix of LD values, with
each row and column corresponding to each of the items in the
set. The diagonal entries should be 1, indicating the LD
between an item in the set and itself is 1.}

\item{num_sims}{An integer value for the number of simulations
to be performed.}

\item{correction}{A logical argument indicating whether a
value of one should be added to the numerator when calculating
the p-value based upon the simulated statistics. By default,
the correction is added. An argument of FALSE will not add one
to the numerator.}

\item{seed}{An integer argument indicating what the random
seed should be set to. This allows for replication of results.
The default is NULL, and a random seed will be set internally.}

\item{verbose}{A logical argument indicating whether periodic
output should be printed. Defaults to FALSE, indicating no
output will be printed.}
}
\value{
A \code{VEGASResult} object with the corresponding
VEGAS results.
}
\description{
Calculates the p-value representing the association of the
set with the phenotype of interest.
}
\details{
This is a helper function to calculate the p-value for a set
of SNPs that reside within a gene set collection. The
correlation among the SNPs is taken into account by the LD
matrix. The resulting p-value is calculated from a null
distribution that is simulated \code{num_sims} times based upon
the specified correlation structure.
}
\author{
Caitlin McHugh \email{mchughc@uw.edu}
}

\name{geneSetName-methods}
\docType{methods}
\alias{geneSetName}
\alias{geneSetName-methods}
\alias{geneSetName,VEGASResult-method}
\alias{geneSetName,GLOSSIResult-method}
\alias{geneSetName,GeneSetResult-method}
\alias{geneSetName,VEGASResultCollection-method}
\alias{geneSetName,GLOSSIResultCollection-method}
\alias{geneSetName,GeneSetResultCollection-method}
\title{ ~~ Methods for Function \code{geneSetName}  ~~}
\description{
This function returns the gene set name for a specified \code{GeneSet} 
or a list of gene set names for a \code{GeneSetCollection}.  
}
\usage{
geneSetName(object)
}
\arguments{
  \item{object}{An object of type \code{GeneSetResult}, \code{VEGASResult}, \code{GLOSSIResult}, 
  	\code{GeneSetResultCollection}, \code{VEGASResultCollection}, or \code{GLOSSIResultCollection} }
}
\section{Methods}{
\describe{
 Defined methods include:
 \item{
      \code{signature(object = "GeneSetResult")}
    }{
      Returns the gene set name for a specified \code{GeneSetResult} object
    }
 \item{
      \code{signature(object = "VEGASResult")}
    }{
      Returns the gene set name for a specified \code{VEGASResult} object
    }
 \item{
      \code{signature(object = "GLOSSIResult")}
    }{
      Returns the gene set name for a specified \code{GLOSSIResult} object
    }
     \item{
      \code{signature(object = "GeneSetResultCollection")}
    }{
      Returns a list of gene set names for a \code{GeneSetResultCollection} object (1 for each set)
    }    
     \item{
      \code{signature(object = "VEGASResultCollection")}
    }{
      Returns a list of gene set names for a \code{VEGASResultCollection} object (1 for each set)
    }
      \item{
      \code{signature(object = "GLOSSIResultCollection")}
    }{
      Returns a list of gene set names for a \code{GLOSSIResultCollection} object (1 for each set)
    }

	}
}
\section{Value}{
\describe{
 Returns the name or a list of names of the gene sets in the corresponding object.
}
}

\seealso{
  \code{\linkS4class{VEGASResult}}-class, \code{pValue}
}
\examples{
	\dontrun{
	geneSetName( vegas(geneSet, assoc_table, ldMatrix) ) 
	}
}
\author{
Caitlin McHugh
}
\keyword{methods}

\name{degreesOfFreedom-methods}
\docType{methods}
\alias{degreesOfFreedom}
\alias{degreesOfFreedom-methods}
\alias{degreesOfFreedom,VEGASResult-method}
\alias{degreesOfFreedom,GLOSSIResult-method}
\alias{degreesOfFreedom,GeneSetResult-method}
\alias{degreesOfFreedom,VEGASResultCollection-method}
\alias{degreesOfFreedom,GLOSSIResultCollection-method}
\alias{degreesOfFreedom,GeneSetResultCollection-method}
\title{ ~~ Methods for Function \code{degreesOfFreedom}  ~~}
\description{
This function returns the degrees of freedom used to calculate p-values from the corresponding \code{GeneSet} 
or a list of degrees of freedom for a \code{GeneSetCollection}.  
}
\usage{
degreesOfFreedom(object)
}
\arguments{
  \item{object}{An object of type \code{GeneSetResult}, \code{VEGASResult}, \code{GLOSSIResult}, 
  	\code{GeneSetResultCollection}, \code{VEGASResultCollection}, or \code{GLOSSIResultCollection} }
}
\section{Methods}{
\describe{
 Defined methods include:
 \item{
      \code{signature(object = "GeneSetResult")}
    }{
      Returns the degrees of freedom for a \code{GeneSetResult} object
    }
 \item{
      \code{signature(object = "VEGASResult")}
    }{
      Returns the degrees of freedom for a \code{VEGASResult} object
    }
 \item{
      \code{signature(object = "GLOSSIResult")}
    }{
      Returns the degrees of freedom for a \code{GLOSSIResult} object
    }
    
    \item{
      \code{signature(object = "GeneSetResultCollection")}
    }{
      Returns a list of degrees of freedom for a \code{GeneSetResultCollection} objects (1 for each set)
    }
     \item{
      \code{signature(object = "VEGASResultCollection")}
    }{
      Returns a list of degrees of freedom for a \code{VEGASResultCollection} objects (1 for each set)
    }
      \item{
      \code{signature(object = "GLOSSIResultCollection")}
    }{
      Returns a list of degrees of freedom for a \code{GLOSSIResultCollection} objects (1 for each set)
    }

	}
}
\section{Value}{
    \describe{
      Returns an integer or a list of integers indicating the degrees of freedom in the corresponding object.
    }
}
\seealso{
  \code{\linkS4class{VEGASResult}}-class, \code{pValue}
}
\examples{
	\dontrun{
	degreesOfFreedom( vegas(geneSet, assoc_table, ldMatrix) ) 
	}
}
\author{
Caitlin McHugh
}
\keyword{methods}

\name{simulatedStats-methods}
\docType{methods}
\alias{simulatedStats}
\alias{simulatedStats-methods}
\alias{simulatedStats,VEGASResult-method}
\alias{simulatedStats,VEGASResultCollection-method}
\title{ ~~ Methods for Function \code{simulatedStats}  ~~}
\description{
This function returns the simulated statistics for a specified \code{GeneSet} 
or a list of simulated statistics for a \code{GeneSetCollection}.  
}
\usage{
simulatedStats(object)
}
\arguments{
  \item{object}{An object of type \code{VEGASResult} or 
  	\code{VEGASResultCollection} }
}
\section{Methods}{
\describe{
 Defined methods include:
 \item{
      \code{signature(object = "VEGASResult")}
    }{
      Returns the simulated statistics for a specified \code{VEGASResult} object
    }    
     \item{
      \code{signature(object = "VEGASResultCollection")}
    }{
      Returns a list of simulated statistics for a \code{VEGASResultCollection} object (1 for each set)
    }
	}
}
\section{Value}{
\describe{
  Returns decimal values of simulated statistics from a VEGAS function call or a list of simulated statistics from a VEGAS function call on a gene set collection.
}
}
\seealso{
  \code{\linkS4class{VEGASResult}}-class, \code{observedStat}
}
\examples{
	\dontrun{
	simulatedStats( vegas(geneSet, assoc_table, ldMatrix) ) 
	}
}
\author{
Caitlin McHugh
}
\keyword{methods}

% Generated by roxygen2 (4.0.1): do not edit by hand
\name{createArrayData}
\alias{createArrayData}
\title{Create a GRanges Object for a GWAS data.frame}
\usage{
createArrayData(arrayData, positionName = NULL,
  chromosomeName = "chromosome", chromosomeNameConvention = "NCBI",
  verbose = TRUE)
}
\arguments{
\item{arrayData}{A data.frame containing array data from
which the \code{GRanges} object will be created. This object is expected
to include the position and chromosome, along with a SNP
identifier and corresponding p-value.}

\item{positionName}{The name of the column in the filepathData
object that holds position data for each probe. By default, this
value is NULL and there are columns Start and End which hold
this information.}

\item{chromosomeName}{The name of the column in the filepathData
object that holds chromosome data for each probe. By default,
this value is \code{chromosome}.}

\item{chromosomeNameConvention}{The naming convention used for the
chromosomes, either \code{NCBI} (default), \code{UCSC}, or a user
specified mapping. By default, this is \code{NCBI}, indicating the autosomes
are stored as integers and the mitochondrial and sex chromosomes are stored
as \code{MT} and \code{X, Y}, respectively. The \code{UCSC} naming
convention appends "chr" to each chromosome, and the mitochondrial coding is
"chrM." Alternatively, a user can supply the mapping from each chromosome
to the UCSC naming convention in the form of a character vector where the
elements are the UCSC names and the corresponding names of the elements are
the current chromosome names. An `NA' chromosome will be mapped to unknown.
For more information, please see the \code{GenomeInfoDb} BioConductor R package.}

\item{verbose}{A logical argument indicating whether output
should be printed. The default is FALSE.}
}
\value{
A GRanges object.
}
\description{
Creates a GRanges object used for SNP set analysis.
}
\details{
This function takes a data.frame and creates a GRanges
object used for SNP set analysis.
}
\author{
Jason Hackney, Jessica Larson, Caitlin McHugh
\email{mchughc@uw.edu}
}
\examples{
    data(geneSetAnalysis)
    head(geneSetAnalysis$arrayData)
    arrayDataGR <- createArrayData(geneSetAnalysis[["arrayData"]], positionName="Position")
}
\keyword{methods}

\name{geneToSNPList}
\alias{geneToSNPList}
\title{Translate a List of Gene Ids to Their Corresponding
SNP Ids}
\usage{
geneToSNPList(geneList, arrayData, genes, maxgap = 20000, quiet = TRUE)
}
\arguments{
\item{geneList}{An object of class \code{GeneSetCollection}
or \code{GeneSet} where geneIds holds the gene Entrez ids.}

\item{arrayData}{A \code{GRanges} object that corresponds to
the probes for which data are present.}

\item{genes}{A \code{GRanges} object holding the gene transcripts
for which the mapping will occur. The ids of the genes should match
the ids that are listed in the \code{geneList} object.}

\item{maxgap}{An integer value indicating the flanking region
around the gene for which SNPs are considered part of the gene.
Default is 20000 (20kb).}

\item{quiet}{A logical variable indicating whether warnings should
be output from the SNP to gene mapping step.}
}
\value{
A \code{GeneSetCollection} object containing all SNP ids that
lie within the genes listed in geneList.
}
\description{
Translates a set of gene ids to their corresponding SNPs.
}
\details{
Translates a GeneSet or GeneSetCollection of gene
Entrez ids to the corresponding SNPs that lie within a
prespecified region of the gene.
}
\author{
Jason Hackney, Jessica Larson,
Caitlin McHugh \email{mchughc@uw.edu}
}
\examples{
    data(geneSetAnalysis)
    head(geneSetAnalysis$arrayData)
    arrayDataGR <- createArrayData(geneSetAnalysis[["arrayData"]], positionName="Position")
    
    library(TxDb.Hsapiens.UCSC.hg19.knownGene)
    genesHg19 <- genes(TxDb.Hsapiens.UCSC.hg19.knownGene)
	geneSets <- geneSetAnalysis[["geneSets"]]
	
	snpsGSC <- geneToSNPList(geneSets, arrayDataGR, genesHg19)
}
\keyword{methods}

% Generated by roxygen2 (4.0.1): do not edit by hand
\name{glossi}
\alias{glossi}
\title{Calculate a Chi-squared Statistic and P-Value for
Independent Items of a Set Using the GLOSSI Method}
\usage{
glossi(pvals, snp.gsc)
}
\arguments{
\item{pvals}{A numerical vector of p-values with names
corresponding to elements listed in the \code{geneIds} slot in
the snp.gsc object.}

\item{snp.gsc}{An object of class \code{GeneSetCollection}
where geneIds holds the items of each set corresponding to the
pvals.}
}
\value{
An object with the corresponding GLOSSI results. If
snp.gsc is a \code{GeneSetCollection} (i.e., multiple sets of
interest), then the corresponding \code{GLOSSIResultCollection}
is returned. If snp.gsc is a \code{GeneSet}, a \code{GLOSSIResult}
object will be returned.
}
\description{
Calculates the p-value representing the association of the
set with the phenotype of interest, assuming all items in the
set are independent, using the GLOSSI method.
}
\details{
This function calculates a p-value for sets of SNPs that
reside within a gene set collection. We calculate the chi-
square p-values and statistic by applying Fisher's
transformation to the observed p-values.
}
\author{
Jason Hackney, Jessica Larson, Caitlin McHugh
\email{mchughc@uw.edu}
}
\references{
Chai, High-Seng and Sicotte, Hughes et al.
GLOSSI: a method to assess the association of genetic loci-sets
with complex diseases.
BMC Bioinformatics, 2009.
}
\examples{
    set.seed(30)
    pvals <- runif(100)
    names(pvals) <- paste0("rs", 1:100)
    snpGS5 <- GeneSet(geneIds=names(pvals)[1:5], setName="set5")
    res <- glossi (pvals, snpGS5)
}

\name{VEGASResult-class}
\Rdversion{1.1}
\docType{class}
\alias{VEGASResult-class}

\title{Class \code{"VEGASResult"}}
\description{
Objects of this class store results from running VEGAS. 

}
\section{Objects from the Class}{
Objects can be created by calls of \code{vegas}.
}
\section{Slots}{
  \describe{
  	\item{\code{geneSetName}:}{Object of class \code{"character"}, the name of the geneSet }
    \item{\code{pValue}:}{Object of class \code{"numeric"}, the p-value }
    \item{\code{degreesOfFreedom}:}{Object of class \code{"integer"}, the degrees of freedom used to calculate the p-value }
    \item{\code{statistic}:}{Object of class \code{"numeric"}, the test statistic for the set }
    \item{\code{simulatedStats}:}{Object of class \code{"vector"}, the simulated statistics for the set used to calculate the p-value }
  }
}
\section{Methods}{
No methods defined with class "VEGASResult" in the signature.
}
\section{Accessors}{
\describe{
In the code snippets below, \code{object} is a VEGASResult object.
    \item{\code{geneSetName(object)}:}{The name of the gene set.}
    \item{\code{pValue(object)}:}{A numeric p-value.}
    \item{\code{degreesOfFreedom(object)}:}{Integer value of the degrees of freedom used in the statistical test.}
    \item{\code{statistic(object)}:}{A numeric value corresponding to the statistic calculated from the test performed.}
}
}
\author{
Caitlin McHugh
}

\seealso{
\code{\linkS4class{VEGASResultCollection}}, \code{\linkS4class{GLOSSIResult}} 
}
\examples{
showClass("VEGASResult")
}
\keyword{classes}

\name{pValue-methods}
\docType{methods}
\alias{pValue}
\alias{pValue-methods}
\alias{pValue,GeneSetResult-method}
\alias{pValue,VEGASResult-method}
\alias{pValue,GLOSSIResult-method}
\alias{pValue,GeneSetResultCollection-method}
\alias{pValue,VEGASResultCollection-method}
\alias{pValue,GLOSSIResultCollection-method}
\title{ ~~ Methods for Function \code{pValue}  ~~}
\description{
This function returns the calculated p-value for a specified \code{GeneSet} 
or a list of p-values for a \code{GeneSetCollection}.  
}
\usage{
pValue(object)
}
\arguments{
  \item{object}{An object of type \code{GeneSetResult}, \code{VEGASResult}, \code{GLOSSIResult}, 
  	\code{GeneSetResultCollection}, \code{VEGASResultCollection}, or \code{GLOSSIResultCollection} }
}
\section{Methods}{
\describe{
 Defined methods include:
 \item{
      \code{signature(object = "VEGASResult")}
    }{
      Returns the calculated p-value for a specified \code{VEGASResult} object
    }
 \item{
      \code{signature(object = "GeneSetResult")}
    }{
      Returns the calculated p-value for a specified \code{GeneSetResult} object
    }
 \item{
      \code{signature(object = "GLOSSIResult")}
    }{
      Returns the calculated p-value for a specified \code{GLOSSIResult} object
    }
    
     \item{
      \code{signature(object = "VEGASResultCollection")}
    }{
      Returns a list of calculated p-values for a \code{VEGASResultCollection} object (1 for each set)
    }
     \item{
      \code{signature(object = "GeneSetResultCollection")}
    }{
      Returns a list of calculated p-values for a \code{GeneSetResultCollection} object (1 for each set)
    }
      \item{
      \code{signature(object = "GLOSSIResultCollection")}
    }{
      Returns a list of calculated p-values for a \code{GLOSSIResultCollection} object (1 for each set)
    }

	}
}
\section{Value}{
\describe{
Returns a decimal p-value corresponding to the gene set or a list of p-values corresponding to the gene set collection.
}
}
\seealso{
  \code{\linkS4class{VEGASResult}}-class, \code{geneSetName}
}
\examples{
	\dontrun{
	pValue( vegas(geneSet, assoc_table, ldMatrix) ) 
	}
}
\author{
Caitlin McHugh
}
\keyword{methods}

% Generated by roxygen2 (4.0.1): do not edit by hand
\name{simulate_chisq}
\alias{simulate_chisq}
\title{Simulate a Chi-squared Statistic from a Distribution
with a Specified Correlation Structure}
\usage{
simulate_chisq(corr_matrix, num_sims)
}
\arguments{
\item{corr_matrix}{A square matrix of correlations.}

\item{num_sims}{An integer value for the number of simulations
to be performed.}
}
\value{
A numeric value corresponding to a chi-squared
statistic simulated from a chi-squared-like distribution with
specified correlation structure and degrees of freedom
corresponding to the number of rows in the correlation matrix.
}
\description{
Simulates a value from a chi-squared distribution with a
specified correlation structure.
}
\details{
This function simulates a value from a chi-squared
distribution with a specified correlation structure and degrees
of freedom corresponding to the number of rows in the
correlation matrix.
}
\author{
Caitlin McHugh \email{mchughc@uw.edu}
}

% Generated by roxygen2 (4.0.1): do not edit by hand
\name{plotPvals}
\alias{plotPvals}
\title{Create a Plot of P-Values Against Number of SNPs
per Set for All Sets in a Collection}
\usage{
plotPvals(set, title = "", xlab = "SNPs per Gene Set", ylab = "PValue",
  ...)
}
\arguments{
\item{set}{An object of class \code{GeneSetResultCollection}.}

\item{title}{A character string containing the title of the plot.
Default is "", a blank title.}

\item{xlab}{A character vector with the label for the x-axis on
the plot. Default is \code{SNPs per Gene Set}.}

\item{ylab}{A character vector holding the label for the y-axis
on the plot. Default is \code{PValue}.}

\item{...}{Further arguments to be passed to the plotting methods,
such as graphical parameters.}
}
\value{
Creates a plot.
}
\description{
Plots association test p-values against number of SNPs per
gene set.
}
\details{
Creates a plot of all p-values against the number of
SNPs in the gene set, for all sets within a collection.
}
\author{
Caitlin McHugh \email{mchughc@uw.edu}
}

\name{statistic-methods}
\docType{methods}
\alias{statistic}
\alias{statistic-methods}
\alias{statistic,GeneSetResult-method}
\alias{statistic,GeneSetResultCollection-method}
\alias{statistic,VEGASResult-method}
\alias{statistic,VEGASResultCollection-method}
\alias{statistic,GLOSSIResult-method}
\alias{statistic,GLOSSIResultCollection-method}
\title{ ~~ Methods for Function \code{statistic}  ~~}
\description{
This function returns the calculated statistic for a specified \code{GeneSet} 
or a list of calculated statistics for a \code{GeneSetCollection}.  
}
\usage{
statistic(object)
}
\arguments{
  \item{object}{An object of type \code{GeneSetResult}, \code{VEGASResult}, \code{GLOSSIResult},
  	 \code{GeneSetResultCollection}, \code{VEGASResultCollection} or \code{GLOSSIResultCollection} }
}
\section{Methods}{
\describe{
 Defined methods include:
 \item{
      \code{signature(object = "GeneSetResult")}
    }{
      Returns the calculated statistic for a specified \code{GeneSetResult} object
    }
      \item{
      \code{signature(object = "GeneSetResultCollection")}
    }{
      Returns a list of calculated statistics for a \code{GeneSetResultCollection} object (1 for each set)
    }
 \item{
      \code{signature(object = "VEGASResult")}
    }{
      Returns the calculated statistic for a specified \code{VEGASResult} object
    }
      \item{
      \code{signature(object = "VEGASResultCollection")}
    }{
      Returns a list of calculated statistics for a \code{VEGASResultCollection} object (1 for each set)
    }
 \item{
      \code{signature(object = "GLOSSIResult")}
    }{
      Returns the calculated statistic for a specified \code{GLOSSIResult} object
    }
      \item{
      \code{signature(object = "GLOSSIResultCollection")}
    }{
      Returns a list of calculated statistics for a \code{GLOSSIResultCollection} object (1 for each set)
    }

	}
}
\section{Value}{
\describe{
Returns decimal values or a list of decimal values corresponding to calculated statistics from a GLOSSI function call.
}
}
\seealso{
  \code{\linkS4class{GLOSSIResult}}-class, \code{pValue}
}
\examples{
	\dontrun{
	statistic( glossi(geneSet, assoc_table, ldMatrix) ) 
	}
}
\author{
Caitlin McHugh
}
\keyword{methods}

\name{geneSetAnalysis}
\docType{data}
\alias{geneSetAnalysis}
\title{Data to run gene set analysis methods}
\description{
	A list including the following components:

	\code{arrayData} -- a data.frame of dimension 119487 rows and 4 columns including P-values, SNP ids, position and chromosome for a GWAS experiment;

	\code{geneSets} -- a GeneSetCollection containing gene Entrez ids;

	\code{ldMat} -- a matrix of dimension 871 x 871 holding linkage disequilibrium values;

	\code{indepSNPs} -- a list containing 302 independent SNP ids;
}
\usage{geneSetAnalysis}
\examples{
    data(geneSetAnalysis)
    head(geneSetAnalysis$arrayData)
}
\keyword{datasets}

\name{GeneSetResult-class}
\Rdversion{1.1}
\docType{class}
\alias{GeneSetResult-class}

\title{Class \code{"GeneSetResult"}}
\description{
Objects of this class store results from running GeneSet methods. 

}
\section{Objects from the Class}{
Objects can be created by calls of \code{glossi} or \code{vegas}.
}
\section{Slots}{
  \describe{
  	\item{\code{geneSetName}:}{Object of class \code{"character"}, the name of the geneSet }
    \item{\code{pValue}:}{Object of class \code{"numeric"}, the p-value }
    \item{\code{degreesOfFreedom}:}{Object of class \code{"integer"}, the degrees of freedom used to calculate the p-value }
    \item{\code{statistic}:}{Object of class \code{"numeric"}, the test statistic for the set }
  }
}
\section{Methods}{
No methods defined with class "GeneSetResult" in the signature.
}
\section{Accessors}{
\describe{
In the code snippets below, \code{object} is a GeneSetResult object.
    \item{\code{geneSetName(object)}:}{The name of the gene set.}
    \item{\code{pValue(object)}:}{A numeric p-value.}
    \item{\code{degreesOfFreedom(object)}:}{Integer value of the degrees of freedom used in the statistical test.}
    \item{\code{statistic(object)}:}{A numeric value corresponding to the statistic calculated from the test performed.}
}
}
\author{
Caitlin McHugh
}

\seealso{
\code{\linkS4class{GeneSetResultCollection}}, \code{\linkS4class{GLOSSIResult}} 
}
\examples{
showClass("GeneSetResult")
}
\keyword{classes}

% Generated by roxygen2 (4.0.1): do not edit by hand
\name{vegas}
\alias{vegas}
\title{Calculate the P-Value for a Set Using the VEGAS Method}
\usage{
vegas(set, assoc_table, ldMatrix, num_sims = 1000, correction = TRUE,
  seed = NULL, verbose = FALSE)
}
\arguments{
\item{set}{A \code{GeneSet} object containing a set of genes
of interest or a \code{GeneSetCollection} object containing a
collection of GeneSets.}

\item{assoc_table}{An object of class \code{GRanges}. This
object should at least contain columns SNP and P which hold SNP
rsIDs and their corresponding association test p-values,
respectively.}

\item{ldMatrix}{A square, symmetric matrix of LD values, with
each row and column corresponding to each of the items in the
set. The diagonal entries should be 1, indicating the LD
between an item in the set and itself is 1.}

\item{num_sims}{A positive integer value for the number of
simulations to be performed. Default is 1000.}

\item{correction}{A logical argument indicating whether a
value of one should be added to the numerator and denominator 
when calculating
the p-value based upon the simulated statistics. By default,
the correction is added and this argument is TRUE.}

\item{seed}{An integer argument indicating what the random
seed should be set to. This allows for replication of results.
The default is NULL, and a random seed will be set internally.}

\item{verbose}{A logical argument indicating whether output
should be printed. The default is FALSE.}
}
\value{
An object with the corresponding VEGAS results. If set
is a \code{GeneSetCollection} (i.e., multiple sets of
interest), then the corresponding \code{VEGASResultCollection}
is returned. If set is a \code{GeneSet}, a \code{VEGASResult}
object will be returned.
}
\description{
Calculates the p-value representing the association of the set
with the phenotype of interest.
}
\details{
This function calculates a p-value for sets of SNPs that
reside within a gene set collection. We calculate the null
distribution by taking into account the observed correlation
among the SNPs and simulating a specified number of statistics
from which the resulting p-value is calculated.
}
\author{
Caitlin McHugh \email{mchughc@uw.edu}
}
\references{
Liu, Jimmy Z. and Mcrae, Allan F. et al.
A Versatile Gene-Based Test for Genome-Wide Association
Studies.
The American Journal of Human Genetics, 2010.
}
